#include <sys/socket.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>

#define MAXBUFSIZE 100
#define NUMFORKS 3



void process_request(int fd)
{
  char address[INET_ADDRSTRLEN], buf[MAXBUFSIZE];
  socklen_t len;
  struct sockaddr_in recaddr;
  int num;

  /* print the remote socket information */

  memset(&recaddr, 0, sizeof(recaddr));
  len = sizeof(recaddr);
  if (getpeername(fd, (struct sockaddr *)&recaddr, &len) < 0)
  {
    perror("Cannot get name");
    exit(1);
  }

  printf("remote machine = %s, port = %hx, %hx.\n", inet_ntop(AF_INET, &recaddr.sin_addr, address, sizeof(address)), recaddr.sin_port, ntohs(recaddr.sin_port));

 
  while (num = read(fd, buf, MAXBUFSIZE)) 
    if (num > 0) 
      write(fd, buf, num);

}


int main(int argc, char * argv[])
{
  int sockfd, rec_sock, i;
  socklen_t len;
  char address[INET_ADDRSTRLEN];
  struct sockaddr_in addr;

  if (argc < 2) 
  {
    fprintf(stderr,"USAGE: %s port.\n", argv[0]);
    exit(-1);
  }

  if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) 
  {
    perror("Cannot get socket");
    exit(1);
  }

  memset(&addr, 0, sizeof(struct sockaddr_in));
  addr.sin_addr.s_addr = INADDR_ANY;
  addr.sin_family = AF_INET;
  addr.sin_port = htons(atoi(argv[1]));

  if (bind(sockfd, (struct sockaddr *)&addr, sizeof(addr)) < 0) 
  {
    perror("bind");
    exit(1);
  }

  
  len = sizeof(addr);
  if (getsockname(sockfd, (struct sockaddr *)&addr, &len) < 0)
  {
    perror("can't get name");
    exit(1);
  }
  
  printf("ip = %s, port = %hd\n", inet_ntop(AF_INET, &addr.sin_addr, address, sizeof(address)), ntohs(addr.sin_port));

  if (listen(sockfd, 5) < 0) 
  {
    perror("listen");
    exit(1);
  }

  for (i=0; i<NUMFORKS; i++) 
  { 
    if (fork() == 0) 
    {
      for(;;)
      {
	if( (rec_sock = accept(sockfd, NULL, NULL)) < 0)
	  if (errno == EINTR)
	    continue;
	  else 
	  {
	    perror("accept");
	    exit(1);
	  }

	process_request(rec_sock);
	close(rec_sock);
      }
    }
  } /* End i<NUMFORKS loop */


  return 0;
}


